import 'package:flutter/material.dart';
import 'package:fluttertoast/fluttertoast.dart';
import 'package:provider/provider.dart';
import 'package:sdk_example/repositories/auth_repository.dart';
import 'package:m4e_sdk/m4e_sdk.dart';
import 'package:sdk_example/screens/auth_screens/verify_otp_code_screen.dart';

class PhoneNumberAuthScreen extends StatelessWidget {
  final _phoneNumbercontroller = TextEditingController();
  final _formmKey = GlobalKey<FormState>();

  void _signInWithPhoneNumber(BuildContext context) {
    final _repository = Provider.of<AuthRepository>(context, listen: false);

    final _phoneNumberForm = M4eAuthPhoneNumberForm(
      id: M4eUniqueId(),
      phoneNumber: M4ePhoneNumber(
        country: 'GHANA',
        number: _phoneNumbercontroller.text,
      ),
    );

    if (!_formmKey.currentState.validate()) return;

    _repository
        .signInWithPhoneNumber(phoneNumberForm: _phoneNumberForm)
        .then((_) {
      if (_repository.state == AuthRepositoryState.error) {
        Fluttertoast.showToast(
            msg: '${_repository.error.code}\n${_repository.error.message}');
      } else {
        // navigate to verification screen
        Navigator.of(context).pushReplacement(
            MaterialPageRoute(builder: (context) => OTPVerificationScreen()));
      }
    });
  }

  Widget _buildPhoneNumberTextField() {
    return TextFormField(
      keyboardType: TextInputType.phone,
      controller: _phoneNumbercontroller,
      textAlignVertical: TextAlignVertical.center,
      decoration: InputDecoration(
        prefixIcon: Icon(Icons.phone),
        hintText: 'Enter phone nummber',
      ),
      validator: (_) {
        return _phoneNumbercontroller.text.isEmpty
            ? 'Enter phone number'
            : null;
      },
    );
  }

  Widget _buildSubmitButton() {
    return Consumer<AuthRepository>(
      builder: (context, repository, __) {
        print(repository.state);

        return FloatingActionButton(
          onPressed: () => _signInWithPhoneNumber(context),
          child: repository.state == AuthRepositoryState.loading
              ? CircularProgressIndicator(
                  backgroundColor: Theme.of(context).primaryColor)
              : Icon(Icons.arrow_forward_ios),
        );
      },
    );
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      body: SafeArea(
        child: Form(
          key: _formmKey,
          child: ListView(
            padding: EdgeInsets.symmetric(horizontal: 30.0),
            children: [
              SizedBox(height: 50.0),
              Text(
                'Login',
                style: Theme.of(context)
                    .primaryTextTheme
                    .headline4
                    .copyWith(color: Colors.black),
              ),
              SizedBox(height: 30.0),
              _buildPhoneNumberTextField(),
              SizedBox(height: 20.0),
              _buildSubmitButton(),
            ],
          ),
        ),
      ),
    );
  }
}
