import 'package:flutter/material.dart';
import 'package:fluttertoast/fluttertoast.dart';
import 'package:provider/provider.dart';
import 'package:sdk_example/repositories/auth_repository.dart';
import 'package:sdk_example/screens/auth_screens/user_account_screen.dart';

class OTPVerificationScreen extends StatelessWidget {
  final _otpCodeController = TextEditingController();
  final _formKey = GlobalKey<FormState>();

  void _signInWithPhoneNumber(BuildContext context) {
    final _repository = Provider.of<AuthRepository>(context, listen: false);

    if (!_formKey.currentState.validate()) return;

    _repository.verifyOTPCode(otpCode: _otpCodeController.text).then((_) {
      if (_repository.state == AuthRepositoryState.error) {
        Fluttertoast.showToast(
            msg: '${_repository.error.code}\n${_repository.error.message}');
      } else {
        // navigate to sign-up screen
        Navigator.of(context).pushReplacement(
            MaterialPageRoute(builder: (context) => UserAccountScreen()));
      }
    });
  }

  Widget _buildOTPCodeTextField() {
    return TextFormField(
      keyboardType: TextInputType.phone,
      controller: _otpCodeController,
      textAlignVertical: TextAlignVertical.center,
      decoration: InputDecoration(
        prefixIcon: Icon(Icons.phone),
        hintText: 'Enter code',
      ),
      validator: (_) {
        return _otpCodeController.text.isEmpty ? 'Enter code' : null;
      },
    );
  }

  Widget _buildSubmitButton() {
    return Consumer<AuthRepository>(
      builder: (context, repository, __) {
        return FloatingActionButton(
          onPressed: () => _signInWithPhoneNumber(context),
          child: repository.state == AuthRepositoryState.loading
              ? CircularProgressIndicator(
                  backgroundColor: Theme.of(context).primaryColor)
              : Icon(Icons.arrow_forward_ios),
        );
      },
    );
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      body: SafeArea(
        child: Form(
          key: _formKey,
          child: ListView(
            padding: EdgeInsets.symmetric(horizontal: 30.0),
            children: [
              SizedBox(height: 50.0),
              Text(
                'Verify Code',
                style: Theme.of(context)
                    .primaryTextTheme
                    .headline4
                    .copyWith(color: Colors.black),
              ),
              Text('Please anter the code sent to your phone'),
              SizedBox(height: 30.0),
              _buildOTPCodeTextField(),
              SizedBox(height: 20.0),
              _buildSubmitButton(),
              SizedBox(height: 20.0),
            ],
          ),
        ),
      ),
    );
  }
}
