import 'package:flutter/material.dart';
import 'package:fluttertoast/fluttertoast.dart';
import 'package:provider/provider.dart';
import 'package:sdk_example/repositories/wallet_repository.dart';

import 'package:m4e_sdk/m4e_sdk.dart';
import 'package:sdk_example/screens/wallet_screens/wallet_form_screen.dart';

class WalletsScreen extends StatefulWidget {
  @override
  _WalletsScreenState createState() => _WalletsScreenState();
}

class _WalletsScreenState extends State<WalletsScreen> {
  @override
  void initState() {
    super.initState();

    Future.microtask(() => _getUserWallets());
  }

  void _navigateToCreateWalletScreen() {
    Navigator.of(context)
        .push(MaterialPageRoute(builder: (context) => WalletFormScreen()))
        .then((value) => _getUserWallets());
  }

  void _getUserWallets() {
    final _repository = Provider.of<WalletRepository>(context, listen: false);

    _repository.getWallets().then((_) {
      if (_repository.state == WalletRepositoryState.error) {
        Fluttertoast.showToast(
            msg: '${_repository.error.code}\n${_repository.error.message}');
      }
    });
  }

  Widget _buildAddWalletFloatingActionButton() {
    return FloatingActionButton(
      onPressed: () => _navigateToCreateWalletScreen(),
      child: Icon(Icons.add),
    );
  }

  Widget _buildWalletBalance(M4eWallet wallet) {
    return Consumer<WalletRepository>(
      builder: (_, repository, __) {
        if (repository.balanceState == WalletRepositoryState.loading ||
            repository.balanceState == WalletRepositoryState.initial) {
          return CircularProgressIndicator();
        }

        if (repository.balanceState == WalletRepositoryState.error) {
          return Icon(Icons.error, color: Colors.red);
        }

        if (repository.balanceState == WalletRepositoryState.success) {
          final _walletBalance = repository.walletBalances?.firstWhere(
              (walletBalance) => walletBalance.wallet.id == wallet.id,
              orElse: () => null);

          return Text(_walletBalance == null
              ? '...'
              : '${_walletBalance.balance.unit} ${_walletBalance.balance.value}');
        }
      },
    );
  }

  Widget _buildWalletsList(M4eWalletList walletList) {
    return ListView.builder(
      itemCount: walletList.wallets.length,
      itemBuilder: (context, index) {
        final _wallet = walletList.wallets[index];

        return ListTile(
          onTap: () {},
          contentPadding: EdgeInsets.symmetric(horizontal: 30.0),
          title: Text(_wallet.walletInfo.label),
          trailing: _buildWalletBalance(_wallet),
        );
      },
    );
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      floatingActionButton: _buildAddWalletFloatingActionButton(),
      appBar: AppBar(
        title: Text('Wallets'),
      ),
      body: Consumer<WalletRepository>(
        builder: (_, repository, __) {
          if (repository.state == WalletRepositoryState.error) {
            return Center(
              child: Padding(
                padding: const EdgeInsets.symmetric(horizontal: 30.0),
                child: Column(
                  mainAxisSize: MainAxisSize.min,
                  children: [
                    Text(
                        '${repository.error.code}\n${repository.error.message}'),
                    RaisedButton(
                      onPressed: () => _getUserWallets(),
                      child: Text('Retry'),
                    ),
                  ],
                ),
              ),
            );
          }

          if (repository.state == WalletRepositoryState.success &&
              repository.wallets != null) {
            return _buildWalletsList(repository.wallets);
          }

          return Center(
            child: CircularProgressIndicator(),
          );
        },
      ),
    );
  }
}
