import 'package:flutter/material.dart';
import 'package:m4e_sdk/m4e_sdk.dart';

class _MobileMoneyWalletProvider extends StatelessWidget {
  final Function(String) onSelectNetworkProvider;
  final String selectedNetworkProvider;

  const _MobileMoneyWalletProvider({
    Key key,
    @required this.onSelectNetworkProvider,
    @required this.selectedNetworkProvider,
  }) : super(key: key);

  @override
  Widget build(BuildContext context) {
    return DropdownButtonFormField(
      hint: Text('Select provider'),
      value: selectedNetworkProvider,
      items: <DropdownMenuItem>[
        DropdownMenuItem(child: Text('MTN Mobile Money'), value: 'MTN'),
      ],
      onChanged: (value) => onSelectNetworkProvider(value),
      validator: (value) {
        return selectedNetworkProvider == null ||
                selectedNetworkProvider.isEmpty
            ? 'Select a network provider'
            : null;
      },
    );
  }
}

class _PhoneNumberTextField extends StatelessWidget {
  final TextEditingController phoneNumberController;

  const _PhoneNumberTextField({Key key, @required this.phoneNumberController})
      : super(key: key);

  @override
  Widget build(BuildContext context) {
    return TextFormField(
      keyboardType: TextInputType.phone,
      controller: phoneNumberController,
      textAlignVertical: TextAlignVertical.center,
      decoration: InputDecoration(
        prefixIcon: Icon(Icons.phone),
        hintText: 'Enter phone nummber',
      ),
      validator: (_) {
        return phoneNumberController.text.isEmpty ? 'Enter phone number' : null;
      },
    );
  }
}

class WalletDetailFormType {
  final Widget child;

  const WalletDetailFormType._({@required this.child});

  factory WalletDetailFormType.mobileMoneyWalletDetail({
    @required Function(String) onSelectNetworkProvider,
    @required String selectedNetworkProvider,
    @required TextEditingController phoneNumberController,
  }) {
    return WalletDetailFormType._(
      child: Column(
        children: [
          _MobileMoneyWalletProvider(
            onSelectNetworkProvider: onSelectNetworkProvider,
            selectedNetworkProvider: selectedNetworkProvider,
          ),
          SizedBox(height: 20.0),
          _PhoneNumberTextField(phoneNumberController: phoneNumberController)
        ],
      ),
    );
  }
}

class WalletDetailForm extends StatelessWidget {
  final String sectionLabel;
  final WalletDetailFormType type;

  const WalletDetailForm(
      {Key key, @required this.type, @required this.sectionLabel})
      : super(key: key);

  @override
  Widget build(BuildContext context) {
    return Column(
      children: [
        ListTile(
          title: Text(
            sectionLabel,
            style: TextStyle(fontWeight: FontWeight.bold),
          ),
        ),
        type.child,
      ],
    );
  }
}
