import 'package:equatable/equatable.dart';
import 'package:flutter/foundation.dart';

import '../../models/models.dart';
import '../m4e_forms.dart';

/// Form parameters required for m4e-sdk transactions
///
class M4eTransactionForm extends Equatable {
  final M4eTransactionWallet from;
  final M4eTransactionWallet to;
  final M4eAmount amount;
  final DateTime at;
  final Map<String, dynamic> meta;

  M4eTransactionForm({
    @required M4eTransactionWallet from,
    @required M4eTransactionWallet to,
    @required M4eAmount amount,
    @required DateTime at,
    @required Map<String, dynamic> meta,
  })  : assert(from != null,
            'M4eTransactionForm [from] wallet should not be null'),
        assert(to != null, 'M4eTransactionForm [to] wallet should not be null'),
        assert(
            from.maybeWhen(
                  m4eWalletProvider: (wallet) => wallet.id,
                  orElse: () => null,
                ) !=
                to.maybeWhen(
                  m4eWalletProvider: (wallet) => wallet.id,
                  orElse: () => null,
                ),
            'M4eTransactionForm [from] and [to] wallets should not be the same'),
        assert(
            amount != null, 'M4eTransactionForm [amount] should not be null'),
        assert(at != null, 'M4eTransactionForm [at] should not be null'),
        assert(meta != null, 'M4eTransactionForm [meta] should not be null'),
        this.from = from,
        this.to = to,
        this.amount = amount,
        this.at = at,
        this.meta = meta;

  @override
  List<Object> get props => [
        from,
        to,
        amount,
        at,
        meta,
      ];

  @override
  String toString() {
    return 'M4eTransactionForm(from: $from, to: ${to.toString()}, amount: $amount, at: $at, meta: $meta)';
  }
}
