import 'package:flutter/foundation.dart';
import 'package:freezed_annotation/freezed_annotation.dart';

import '../../models/models.dart';
import '../m4e_forms.dart';
import '../../core/core.dart';

part 'm4e_transaction_wallet.freezed.dart';

@freezed
abstract class M4eTransactionWallet with _$M4eTransactionWallet {
  const factory M4eTransactionWallet.m4eWalletProvider(
      {@required M4eWallet wallet}) = M4eWalletProvider;
  const factory M4eTransactionWallet.mobileMoneyWalletProvider({
    @required M4eNetworkProviderType provider,
    @required M4ePhoneNumber phoneNumber,
  }) = MobileMoneyWalletProvider;
}

extension MobileMoneyWalletProviderXtension on MobileMoneyWalletProvider {
  M4eWalletForm toWalletForm({
    @required M4eUniqueId walletId,
    @required String label,
    @required M4eCurrencyType currencyType,
  }) {
    assert(walletId != null,
        'MobileMoneyWalletProvider [walletId] should not be null');
    assert(
        label != null, 'MobileMoneyWalletProvider [label] should not be null');
    assert(currencyType != null,
        'MobileMoneyWalletProvider [currencyType] should not be null');

    return M4eWalletForm(
      id: walletId,
      label: label,
      currency: currencyType,
      type: M4eWalletType.MOBILE_MONEY,
      detail: M4eWalletDetailType.mobileMoneyWalletDetail(
        provider: this.provider,
        phoneNumber: this.phoneNumber,
      ),
    );
  }
}
