import 'package:equatable/equatable.dart';
import 'package:flutter/cupertino.dart';
import 'package:shortid/shortid.dart';

import '../../core/core.dart';
import '../../models/models.dart';

class M4eUserForm extends Equatable {
  final String shortName;
  final String firstName;
  final String lastName;
  final M4ePhoneNumber phoneNumber;
  final int joined;
  final String publicKey;

  // TODO: remove public-key form params in time
  M4eUserForm({
    @required String firstName,
    @required String lastName,
    @required M4ePhoneNumber phoneNumber,
  })  : assert(firstName != null, 'M4eUserForm [firstName] should not be null'),
        assert(lastName != null, 'M4eUserForm [lastName] should not be null'),
        assert(phoneNumber != null,
            'M4eUserForm [phoneNumber] should not be null'),
        this.shortName = '${firstName}_$lastName-${shortid.generate()}',
        this.firstName = firstName,
        this.lastName = lastName,
        this.phoneNumber = phoneNumber,
        this.joined = DateTime.now().toTimestampSinceEpoch(),
        this.publicKey = 'demo-public-key';

  Map<String, dynamic> toJson() {
    return {
      'user': {
        'shortName': this.shortName,
        'firstName': this.firstName,
        'lastName': this.lastName,
        'phone': this.phoneNumber.toJson(),
        'joined': this.joined,
      },
      'pubKey': this.publicKey
    };
  }

  @override
  List<Object> get props =>
      [shortName, firstName, lastName, phoneNumber, joined, publicKey];

  @override
  String toString() {
    return 'M4eUserForm(shortName: $shortName, firstName: $firstName, lastName: $lastName, phoneNumber: $phoneNumber, joined: $joined, publicKey: $publicKey, publicKey]: $publicKey])';
  }
}
