import 'package:equatable/equatable.dart';
import 'package:flutter/foundation.dart';

import '../core/core.dart';
import 'models.dart';

class M4eUser extends Equatable {
  final M4eUniqueId id;
  final String shortName;
  final String firstName;
  final String lastName;
  final M4ePhoneNumber phoneNumber;
  final DateTime joined;

  const M4eUser({
    @required this.id,
    @required this.shortName,
    @required this.firstName,
    @required this.lastName,
    @required this.phoneNumber,
    @required this.joined,
  });

  // A valid [UniqueId] is passed with the decoded map from [UserApi] else it is null
  factory M4eUser.fromJson(Map<String, dynamic> json) {
    // TODO: consider changing [user][user] to something else
    final _data = json['user'];

    return M4eUser(
      id: M4eUniqueId.fromUniqueString(_data['id']),
      shortName: _data['user']['shortName'],
      firstName: _data['user']['firstName'],
      lastName: _data['user']['lastName'],
      phoneNumber: M4ePhoneNumber.fromJson(_data['user']['phone']),
      joined:
          DateTimeConverter.fromTimestampSinceEpoch(_data['user']['joined']),
    );
  }

  Map<String, dynamic> toJson() {
    return {
      'user': {
        'id': this.id.toString(),
        'user': {
          'shortName': this.shortName,
          'firstName': this.firstName,
          'lastName': this.lastName,
          'phone': this.phoneNumber.toJson(),
          'joined': this.joined.toTimestampSinceEpoch(),
        }
      },
    };
  }

  @override
  List<Object> get props =>
      [id, shortName, firstName, lastName, phoneNumber, joined];

  @override
  String toString() {
    return 'M4eUser(id: $id, shortName: $shortName, firstName: $firstName, lastName: $lastName, phoneNumber: $phoneNumber, joined: $joined)';
  }
}
