import 'package:data_connection_checker/data_connection_checker.dart';
import 'package:flutter_test/flutter_test.dart';
import 'package:m4e_sdk/src/core/core.dart';
import 'package:m4e_sdk/src/data/api/apis.dart';
import 'package:m4e_sdk/src/data/services/services.dart';
import 'package:m4e_sdk/src/data/services/utils/utils.dart';
import 'package:m4e_sdk/src/m4e_forms/m4e_forms.dart';
import 'package:m4e_sdk/src/models/models.dart';
import 'package:matcher/matcher.dart';
import 'package:mockito/mockito.dart';

import 'test_models/wallet_test_model.dart';
// import 'package:matcher/matcher.dart';

class MockWalletApi extends Mock implements WalletApi {}

class MockTransactionApi extends Mock implements TransactionApi {}

class MockDataConnectionChecker extends Mock implements DataConnectionChecker {}

main() {
  M4eTransactionService _m4eTransactionService;

  MockWalletApi _mockWalletApi;
  MockTransactionApi _mockTransactionApi;
  MockDataConnectionChecker _mockDataConnectionChecker;
  TransactionServiceUtil _transactionServiceUtil;

  setUp(() {
    _mockWalletApi = MockWalletApi();
    _mockTransactionApi = MockTransactionApi();
    _mockDataConnectionChecker = MockDataConnectionChecker();

    // transaction-util
    _transactionServiceUtil = TransactionServiceUtil(walletApi: _mockWalletApi);

    _m4eTransactionService = M4eTransactionService(
      walletApi: _mockWalletApi,
      transactionApi: _mockTransactionApi,
      connectionChecker: _mockDataConnectionChecker,
    );
  });

  final _uuid =
      M4eUniqueId.fromUniqueString('46e87882-4ff8-4770-9846-870fbbcc35e0');
  final _fromUuId =
      M4eUniqueId.fromUniqueString('46e87882-4ff8-4770-9846-870fbbcc35e0');
  final _toUuId =
      M4eUniqueId.fromUniqueString('9e8b5ecc-9f7e-4e06-ac3d-05b11a91da88');

  final _phoneNumber = M4ePhoneNumber(number: '0541234567', country: 'GHANA');

  final _m4eWalletDetail = M4eWalletDetail(id: _toUuId);
  final _mobileMoneyWalletDetail = MobileMoneyWalletDetail(
      provider: M4eNetworkProviderType.MTN, phoneNumber: _phoneNumber);

  final _tM4eToM4eTransaction = M4eTransactionForm(
    from: M4eTransactionWallet.m4eWalletProvider(
      wallet: M4eWallet(
        id: _fromUuId,
        walletInfo: M4eWalletInfo(
          label: 'W1',
          currency: M4eCurrencyType.GHS,
          type: M4eWalletType.M4E,
        ),
      ),
    ),
    to: M4eTransactionWallet.m4eWalletProvider(
      wallet: M4eWallet(
        id: _toUuId,
        walletInfo: M4eWalletInfo(
          label: 'W2',
          currency: M4eCurrencyType.GHS,
          type: M4eWalletType.M4E,
        ),
      ),
    ),
    amount: M4eAmount(currency: M4eCurrencyType.GHS, value: 50.0),
    at: DateTime.now(),
    meta: <String, dynamic>{
      'callbackUrl': 'www.google.com',
      'purpose': 'payment',
      'data': 'test data',
    },
  );

  final _tM4eToMTNTransaction = M4eTransactionForm(
    from: M4eTransactionWallet.m4eWalletProvider(
      wallet: M4eWallet(
        id: _fromUuId,
        walletInfo: M4eWalletInfo(
          label: 'W1',
          currency: M4eCurrencyType.GHS,
          type: M4eWalletType.M4E,
        ),
      ),
    ),
    to: M4eTransactionWallet.mobileMoneyWalletProvider(
      provider: M4eNetworkProviderType.MTN,
      phoneNumber: _phoneNumber,
    ),
    amount: M4eAmount(currency: M4eCurrencyType.GHS, value: 50.0),
    at: DateTime.now(),
    meta: <String, dynamic>{
      'callbackUrl': 'www.google.com',
      'purpose': 'payment',
      'data': 'test data',
    },
  );

  final _tTransactionResult = M4eTransactionResult(
    from: _fromUuId,
    to: _toUuId,
    amount: M4eAmount(
      currency: M4eCurrencyType.GHS,
      value: 50.0,
    ),
    at: DateTime.now(),
    txType: M4eTransactionType.parse('CASH_IN'),
    meta: <String, dynamic>{
      'callbackUrl': 'www.google.com',
      'purpose': 'payment',
      'data': 'test data',
    },
  );

  final _userId =
      M4eUniqueId.fromUniqueString('46e87882-4ff8-4770-9846-870fbbcc35e0');

  final _transactionLogs = [
    M4eTransaction(
      id: _uuid,
      from: M4eWallet(
        id: _fromUuId,
        walletInfo: M4eWalletInfo(
          label: 'W1',
          currency: M4eCurrencyType.GHS,
          type: M4eWalletType.M4E,
        ),
      ),
      to: M4eWallet(
        id: _toUuId,
        walletInfo: M4eWalletInfo(
          label: 'W2',
          currency: M4eCurrencyType.GHS,
          type: M4eWalletType.M4E,
        ),
      ),
      amount: M4eAmount(currency: M4eCurrencyType.parse('GHC'), value: 50.0),
      at: DateTime.now(),
      type: M4eTransactionType.cashIn,
      meta: <String, dynamic>{
        'callbackUrl': 'www.google.com',
        'purpose': 'payment',
        'data': 'test data',
      },
    ),
  ];

  final _transactionList = M4eTransactionList(transactions: _transactionLogs);

  // final _fromWallet = await _tM4eToM4eTransaction.from.map(
  //   m4eWalletProvider: (provider) {
  //     return _transactionServiceUtil.mobileMoneyTransactionMapper(
  //       userId: _userId,
  //       transaction: _tM4eToM4eTransaction,
  //       walletProvider: _tM4eToM4eTransaction.from,
  //     );
  //   },
  //   mobileMoneyWalletProvider: (provider) {
  //     return _transactionServiceUtil.m4eProviderTransactionMapper(
  //       walletProvider: _tM4eToM4eTransaction.from,
  //     );
  //   },
  // );

  // final _toWallet = await _tM4eToM4eTransaction.from.map(
  //   m4eWalletProvider: (provider) {
  //     return _transactionServiceUtil.mobileMoneyTransactionMapper(
  //       userId: _userId,
  //       transaction: _tM4eToM4eTransaction,
  //       walletProvider: _tM4eToM4eTransaction.to,
  //     );
  //   },
  //   mobileMoneyWalletProvider: (provider) {
  //     return _transactionServiceUtil.m4eProviderTransactionMapper(
  //       walletProvider: _tM4eToM4eTransaction.to,
  //     );
  //   },
  // );

  // create [InternalTransactionForm]
  // final _m4eToM4eModifiedTransaction = InternalTransactionForm(
  //   from: _fromWallet,
  //   to: _toWallet,
  //   amount: _tM4eToM4eTransaction.amount,
  //   at: _tM4eToM4eTransaction.at,
  //   meta: _tM4eToM4eTransaction.meta,
  // );

  group('m4e-transaction-test initiateTransaction |', () {
    test('should verify device connectivity status', () async {
      // arrange
      when(_mockDataConnectionChecker.hasConnection)
          .thenAnswer((_) async => true);
      when(_mockTransactionApi.initiateTransaction(any))
          .thenAnswer((_) async => _tTransactionResult);

      // act
      await _m4eTransactionService.initiateTransaction(
          _fromUuId, _tM4eToM4eTransaction);

      // assert
      verify(_mockDataConnectionChecker.hasConnection);
    });

    group('isConnected |', () {
      final _m4eToMTNTransactionResult = InternalTransactionForm(
        from: _tM4eToM4eTransaction.from.maybeMap(
            m4eWalletProvider: (provider) => provider.wallet,
            orElse: () => null),
        to: _tM4eToM4eTransaction.to.maybeMap(
            m4eWalletProvider: (provider) => provider.wallet,
            orElse: () => null),
        amount: _tM4eToM4eTransaction.amount,
        at: _tM4eToM4eTransaction.at,
        meta: _tM4eToM4eTransaction.meta,
      );

      final _m4eToM4eTransactionResult = InternalTransactionForm(
        from: _tM4eToM4eTransaction.from.maybeMap(
            m4eWalletProvider: (provider) => provider.wallet,
            orElse: () => null),
        to: _tM4eToM4eTransaction.to.maybeMap(
            m4eWalletProvider: (provider) => provider.wallet,
            orElse: () => null),
        amount: _tM4eToM4eTransaction.amount,
        at: _tM4eToM4eTransaction.at,
        meta: _tM4eToM4eTransaction.meta,
      );

      setUp(() {
        when(_mockDataConnectionChecker.hasConnection)
            .thenAnswer((_) async => true);
      });

      test('should return [M4eTransactionResult] on success call', () async {
        // arrange
        when(_mockTransactionApi.initiateTransaction(any))
            .thenAnswer((_) async => _tTransactionResult);

        // act
        final _result = await _m4eTransactionService.initiateTransaction(
            _fromUuId, _tM4eToM4eTransaction);

        // assert
        expect(_result, equals(_tTransactionResult));
      });

      test(
          'should return [M4eTransactionResult] from TransactionApi on success call',
          () async {
        // arrange
        when(_mockTransactionApi.initiateTransaction(any))
            .thenAnswer((_) async => _tTransactionResult);

        // act
        await _m4eTransactionService.initiateTransaction(
            _fromUuId, _tM4eToM4eTransaction);

        // assert
        // final _internalTransactionForm =
        //     await _transactionServiceUtil.m4eProviderTransactionMapper(
        //         walletProvider: _tM4eToM4eTransaction.from);

        verify(_mockTransactionApi
            .initiateTransaction(_m4eToM4eTransactionResult));
      });

      group('mobileMoneyProviderTransactionMapper', () {
        final _composedMobileMoneyWalletSearchForm =
            MobileMoneyProviderSearchForm(
          provider: _tM4eToMTNTransaction.to.maybeMap(
            mobileMoneyWalletProvider: (reciepient) => reciepient.provider,
            orElse: () => null,
          ),
          phoneNumber: _tM4eToMTNTransaction.to.maybeMap(
            mobileMoneyWalletProvider: (reciepient) => reciepient.phoneNumber,
            orElse: () => null,
          ),
        );

        final _toWalletForm = _tM4eToMTNTransaction.to.maybeMap(
          mobileMoneyWalletProvider: (reciepient) => reciepient.toWalletForm(
            walletId: WalletTestModel.uniqueId,
            label: 'test',
            currencyType: _tM4eToMTNTransaction.amount.currency,
          ),
          orElse: () => null,
        );

        final _m4eToMTNTransactionResult = M4eTransactionResult(
          from: _fromUuId,
          to: _tM4eToMTNTransaction.to.maybeMap(
            m4eWalletProvider: (reciepient) => reciepient.wallet.id,
            orElse: () => null,
          ),
          amount: _tTransactionResult.amount,
          at: _tTransactionResult.at,
          txType: M4eTransactionType.cashOut,
          meta: _tTransactionResult.meta,
        );

        test(
            'should call [WalletApi] `queryUserWallets` method with appropriate parameters when a [MobileMoneyWalletProvider] transaction is made',
            () async {
          // arrange
          when(_mockWalletApi.queryUserWallets(any))
              .thenAnswer((_) async => WalletTestModel.uniqueId);
          when(_mockWalletApi.runSearchQuery(any)).thenAnswer((_) async =>
              WalletTestModel.tWalletListAnonymousSearchByPhoneNumber);

          // act
          await _m4eTransactionService.initiateTransaction(
              _userId, _tM4eToMTNTransaction);

          // assert
          // final _internalTransactionForm =
          //     await _transactionServiceUtil.m4eProviderTransactionMapper(
          //         walletProvider: _tM4eToM4eTransaction.from);

          verify(_mockWalletApi
              .queryUserWallets(_composedMobileMoneyWalletSearchForm));
        });

        test(
            'should call [WalletApi] `runSearchQuery` method to run search-query-id with appropriate parameters when a [MobileMoneyWalletProvider] transaction is made',
            () async {
          // arrange
          when(_mockWalletApi.queryUserWallets(any))
              .thenAnswer((_) async => WalletTestModel.uniqueId);
          when(_mockWalletApi.runSearchQuery(any)).thenAnswer((_) async =>
              WalletTestModel.tWalletListAnonymousSearchByPhoneNumber);

          // act
          await _m4eTransactionService.initiateTransaction(
              _userId, _tM4eToMTNTransaction);

          // assert
          verify(_mockWalletApi.runSearchQuery(WalletTestModel.uniqueId));
        });

        test(
            'should call [WalletApi] `createWallet` method to create a wallet if none was fetched from search-query',
            () async {
          // arrange
          when(_mockWalletApi.queryUserWallets(any))
              .thenAnswer((_) async => WalletTestModel.uniqueId);
          when(_mockWalletApi.runSearchQuery(any))
              .thenAnswer((_) async => WalletTestModel.tWalletListEmpty);

          // act
          await _m4eTransactionService.initiateTransaction(
              _userId, _tM4eToMTNTransaction);

          // assert
          final _toWalletForm = _tM4eToMTNTransaction.to.maybeMap(
            mobileMoneyWalletProvider: (reciepient) => reciepient.toWalletForm(
              walletId: WalletTestModel.uniqueId,
              label: 'test',
              currencyType: _tM4eToMTNTransaction.amount.currency,
            ),
            orElse: () => null,
          );

          verify(_mockWalletApi
              .queryUserWallets(_composedMobileMoneyWalletSearchForm));
          verify(_mockWalletApi.runSearchQuery(WalletTestModel.uniqueId));

          // TODO: fix
          // verify(_mockWalletApi.createWallet(_userId, _toWalletForm));
        });

        test(
            'should map [M4eWallet] to [InternalTransactionForm] `to` object when return from search',
            () async {
          // arrange
          when(_mockWalletApi.queryUserWallets(any))
              .thenAnswer((_) async => WalletTestModel.uniqueId);
          when(_mockWalletApi.runSearchQuery(any)).thenAnswer((_) async =>
              WalletTestModel.tWalletListAnonymousSearchByPhoneNumber);
          when(_mockTransactionApi.initiateTransaction(any))
              .thenAnswer((_) async => _m4eToMTNTransactionResult);

          // act
          await _m4eTransactionService.initiateTransaction(
              _userId, _tM4eToMTNTransaction);

          // assert
          final _recipientWallet =
              await _transactionServiceUtil.mobileMoneyTransactionMapper(
            userId: _userId,
            transaction: _tM4eToMTNTransaction,
            walletProvider: _tM4eToMTNTransaction.to,
          );

          final _internalTransactionForm = InternalTransactionForm(
            from: _tM4eToMTNTransaction.from.maybeMap(
                m4eWalletProvider: (provider) => provider.wallet,
                orElse: () => null),
            to: _recipientWallet,
            amount: _tM4eToMTNTransaction.amount,
            at: _tM4eToMTNTransaction.at,
            meta: _tM4eToMTNTransaction.meta,
          );

          verify(_mockWalletApi
              .queryUserWallets(_composedMobileMoneyWalletSearchForm));
          verify(_mockWalletApi.runSearchQuery(WalletTestModel.uniqueId));
          verify(_mockTransactionApi
              .initiateTransaction(_internalTransactionForm));
        });

        test(
            'should call [TransactionApi] to initiate transaction with appropriate parameters',
            () async {
          // arrange
          when(_mockWalletApi.queryUserWallets(any))
              .thenAnswer((_) async => WalletTestModel.uniqueId);
          when(_mockWalletApi.runSearchQuery(any))
              .thenAnswer((_) async => WalletTestModel.tWalletListEmpty);
          when(_mockWalletApi.createWallet(any, any))
              .thenAnswer((_) async => _toUuId);

          // act
          await _m4eTransactionService.initiateTransaction(
              _userId, _tM4eToMTNTransaction);

          // final _internalTransactionForm = await _transactionServiceUtil
          //     .mobileMoneyProviderTransactionMapper(
          //         _userId, _tM4eToMTNTransaction);

          // final _reciepientToWalletFormType =
          //     _transactionServiceUtil.mobileMoneyWalletProviderToFormWalletType(
          //   reciepient: _tM4eToMTNTransaction.to,
          //   walletId: WalletTestModel.uniqueId,
          //   label: 'test',
          //   currencyType: M4eCurrencyType.GHS,
          // );

          // // print(_toWalletForm == _reciepientToWalletFormType);

          // TODO: fix

          print(_mockWalletApi.createWallet(_userId, _toWalletForm) ==
              _mockWalletApi.createWallet(_userId, _toWalletForm));

          verify(_mockWalletApi.createWallet(_userId, _toWalletForm));
          verify(_mockWalletApi
              .queryUserWallets(_composedMobileMoneyWalletSearchForm));
          verify(_mockWalletApi.runSearchQuery(WalletTestModel.uniqueId));
          // verify(_mockTransactionApi
          //     .initiateTransaction(_internalTransactionForm));

          // expect(_result, equals(_expectedResult));
        });

        test(
            'should return [TransactionResult] with `to` object as the uuid of the created wallet',
            () async {
          // arrange
          when(_mockWalletApi.queryUserWallets(any))
              .thenAnswer((_) async => WalletTestModel.uniqueId);
          when(_mockWalletApi.runSearchQuery(any))
              .thenAnswer((_) async => WalletTestModel.tWalletListEmpty);
          when(_mockWalletApi.createWallet(any, any))
              .thenAnswer((_) async => _toUuId);
          when(_mockTransactionApi.initiateTransaction(any))
              .thenAnswer((_) async => _m4eToMTNTransactionResult);

          // act
          final _result = await _m4eTransactionService.initiateTransaction(
              _userId, _tM4eToMTNTransaction);

          // final _internalTransactionForm = await _transactionServiceUtil
          //     .mobileMoneyProviderTransactionMapper(
          //         _userId, _tM4eToMTNTransaction);

          // TODO: fix
          print(_mockWalletApi.createWallet(_userId, _toWalletForm) ==
              _mockWalletApi.createWallet(_userId, _toWalletForm));

          // assert
          verify(_mockWalletApi
              .queryUserWallets(_composedMobileMoneyWalletSearchForm));
          verify(_mockWalletApi.runSearchQuery(WalletTestModel.uniqueId));
          verify(_mockWalletApi.createWallet(_userId, _toWalletForm));
          // verify(_mockTransactionApi
          //     .initiateTransaction(_internalTransactionForm));

          expect(_result, equals(_m4eToMTNTransactionResult));
        });
      });
    });

    group('isDisconnected |', () {
      setUp(() {
        when(_mockDataConnectionChecker.hasConnection)
            .thenAnswer((_) async => false);
      });

      test(
          'should throw [ServerException] with [NO_INTERNET_CONNECTION] error code',
          () async {
        // arrange
        when(_mockTransactionApi.initiateTransaction(any))
            .thenThrow(M4eExceptionMessages.kNoInternetConnectionException);

        // act
        final _call = _m4eTransactionService.initiateTransaction;

        // assert
        expect(() => _call(_fromUuId, _tM4eToM4eTransaction),
            throwsA(TypeMatcher<M4eServerException>()));
      });
    });
  });

  group('getUserTransactions | ', () {
    test('should verify device connectivity status', () async {
      // arrange
      when(_mockDataConnectionChecker.hasConnection)
          .thenAnswer((_) async => true);
      when(_mockTransactionApi.getUserTransactions(any))
          .thenAnswer((_) async => _transactionList);

      // act
      await _m4eTransactionService.getUserTransactions(_userId);

      // assert
      verify(_mockDataConnectionChecker.hasConnection);
    });

    group('isConnected', () {
      setUp(() {
        when(_mockDataConnectionChecker.hasConnection)
            .thenAnswer((_) async => true);
      });

      test(
          'should call TransactionApi to get user-transaction-logs with userId',
          () async {
        // arrange
        when(_mockTransactionApi.getUserTransactions(any)).thenAnswer(
            (_) async => M4eTransactionList(transactions: _transactionLogs));

        // act
        await _m4eTransactionService.getUserTransactions(_userId);

        // assert
        verify(_mockTransactionApi.getUserTransactions(_userId));
      });

      test('should return TransactionLogList on success', () async {
        // arrange
        when(_mockTransactionApi.getUserTransactions(any)).thenAnswer(
            (_) async => M4eTransactionList(transactions: _transactionLogs));

        // act
        final _result =
            await _m4eTransactionService.getUserTransactions(_userId);

        // assert
        verify(_mockTransactionApi.getUserTransactions(_userId));
        expect(_result,
            equals(M4eTransactionList(transactions: _transactionLogs)));
      });
    });

    group('isDisconnected', () {
      setUp(() {
        when(_mockDataConnectionChecker.hasConnection)
            .thenAnswer((_) async => false);

        test(
            'should throw a ServerException when no internet connection is detected',
            () async {
          // arrange
          when(_mockTransactionApi.getUserTransactions(any))
              .thenThrow(M4eExceptionMessages.kNoInternetConnectionException);

          // act
          final _call = _m4eTransactionService.getUserTransactions;

          // assert
          expect(
              () => _call(_userId), throwsA(TypeMatcher<M4eServerException>()));
        });
      });
    });
  });
}
